## Time-stamp: <30-08-2007 13:51:43 ste PC69>

########################################################
## This code defines several class and methods for the FLUSH
## algorithm
##
## Author: Stefano Calza
## First version: 28 May 2007
##
## Note: radical library re-design from version 0.1.0
##
#######################################################

setClass("RA",representation(data = "data.frame", ProbeID = "character", keep = "matrix",
                             fitted = "matrix",
                             fdr = "numeric", cdfName = "character"),
         prototype = prototype(data = data.frame(), ProbeID = character(), keep = matrix(),
           fitted = matrix(),
           fdr = numeric(), cdfName = character())
         ,package="FLUSH")

setClass("FLUSH",representation(exprs = "matrix", ProbeID = "character", RA = "data.frame",
                                fdr = "numeric",
                                onlyExprs = "logical", keep = "logical", cdfName = "character"),
         prototype = prototype(exprs = matrix(), ProbeID = character(),RA = data.frame(),
           fdr = numeric(), onlyExprs = logical(), keep = logical(), cdfName = character())
         ,package="FLUSH")

setGeneric("Flush",function(object,RA, check = TRUE, ...)
           standardGeneric("Flush")
           ,package="FLUSH")

setGeneric("FlushSet",function(object,proportion = 0.6, delta = NULL, lambda = NULL,
                               df = 10,...)
           standardGeneric("FlushSet")
           ,package="FLUSH")

setGeneric("fitRA",function(object,verbose=FALSE)
           standardGeneric("fitRA")
           ,package="FLUSH")

setGeneric("RAplot",function(object,...)
           standardGeneric("RAplot")
           ,package="FLUSH")



#############
## CHANGES ##
#############

##  ??? slot keep & fitted set to matrix in order to hold more than one RQ fit varying proportion
##  28May07: updated to deal with ExpressionSet class

setMethod("fitRA","AffyBatch",
          function (object,verbose=FALSE)
          {
            
##             if(!inherits(object,"AffyBatch"))
##               stop("An object of class AffyBatch is needed as input\n")
            
            ProbeNames <- probeNames(object)
            GeneNames <- unique(ProbeNames)
            nGenes <- length(GeneNames)
            tabProbes <- table(ProbeNames)
            nsamples <- nrow(pData(object))
            
            PM <- log2(matrix(.C("IdealMM_correct", as.double(pm(object)), 
                                 as.double(mm(object)), as.integer(length(ProbeNames)), 
                                 as.integer(nsamples), as.character(ProbeNames),
                                 PACKAGE="affyPLM")[[1]], 
                              ncol = nsamples))
            
            out <- matrix(0, nrow = nGenes, ncol = 2)
            colnames(out) <- c("logResSD", "sqrtArrays")

            
            rownames(out) <- GeneNames
            if(verbose)
              cat("Fitting models\n")
                        
            ## were to start reading PM each iteration
            start.read <- cumsum(c(0,tabProbes[-length(tabProbes)]))+1
            stop.read <- start.read + tabProbes - 1
            
            n.b <- nsamples - 1
            
            i = 1
            X <- make.design(nsamples=nsamples,nprobes=tabProbes[i])
            nr <- nrow(X)
            p <- ncol(X)
            start.p <- tabProbes[i] + 1
            
            y = PM[start.read[i]:stop.read[i],]
            
            ln <- length(y)
            out[i, ] <- RA.fit(X = X, y = y, nr = nr, p = p, ln = ln, 
                               start.p, n.b)
            
            for (i in 2:nGenes)
              {
                if(verbose)
                  {
                    if (!i%%1000) 
                      cat(i, "\r")
                  }
                if (tabProbes[i] != tabProbes[i - 1]) {
                  
                  X <- make.design(nsamples=nsamples,nprobes=tabProbes[i])
                  start.p <- tabProbes[i] + 1
                  
                  nr <- nrow(X)
                  p <- ncol(X)
                  ln <- nsamples*tabProbes[i]
                  
                }
                
                y = PM[start.read[i]:stop.read[i],]
                
                out[i, ] <- RA.fit(X = X, y = y, nr = nr, p = p, ln = ln, 
                                   start.p, n.b)
              }
            if(verbose)
              cat("\n")

            out[,"logResSD"] <- log(out[,"logResSD"])
            out[,"sqrtArrays"] <- sqrt(out[,"sqrtArrays"])
            out <- as.data.frame(out)
            
            ans <- new("RA",data = out, ProbeID = GeneNames, cdfName = object@cdfName)
            
            return(ans)
            
          }
          )


## CHANGES 29Aug06: add which option for the case of more than one proportion used
##         01May07: change to ExpressionSet -> Bioconductor 2.0 compatible
##         30Aug07: corrected bug in Flush (object@exprs not valid from BioC 2)
#                  Flush(...,onlyExprs=TRUE) now creates an object of class ExpressionSet

setMethod("Flush","ExpressionSet",
          function(object, RA,  check = TRUE, onlyExprs = TRUE, which = 1)
          {
            if(!length(RA@keep))
              stop("Need first use FlushSet on RA object")

            if(which > ncol(RA@keep))
              stop(paste("Arg which must be [0,",ncol(RA@keep),"]",sep=""))
               
            keep <- RA@keep[,which]
               
            if(check)
              if(!all.equal(featureNames(object),RA@ProbeID))
                if(!all(feaureNames(object) %in% RA@ProbeID))
                  stop("Object gene names don't correspond to RA Probes ID. Please check!")
                else
                  {
                    keep <- keep[match(featureNames(object),RA@ProbeID)]
                    RA@data <- RA@data[match(featureNames(object),RA@ProbeID)]
                  }

            out <- exprs(object)[keep,]
            probeID <- featureNames(object)[keep]
            
            if(onlyExprs)
              {
                
                ans <- new("ExpressionSet",exprs=out,phenoData = phenoData(object),
                           annotation=annotation(object))
                description(ans) <- description(object)
              }
            else
              {
                ares <- data.frame(RA@data[,c("logResSD","sqrtArrays")],fitted=RA@fitted)

                ans <- new("FLUSH",exprs = out, ProbeID = probeID, RA = ares,
                           onlyExprs = onlyExprs,
                           keep = keep, cdfName = RA@cdfName)
              }

            
            return(ans)
            
          })

setMethod("Flush","matrix",
          function(object, RA, check = FALSE,...)
          {
            if(!length(RA@keep))
              stop("Need first use FlushSet on RA object")

            if(check)
              warning("No check defined for object of class matrix")
            
            out <- object[RA@keep,]
            return(out)
            
          })



## CHANGES: 29Aug06 specify better the args delta and lambda
##          29Aug06 allow to use more than 1 value for proportion
##          1May07 allow to specify the df of the B-spline

setMethod("FlushSet","RA",
          function(object, proportion = 0.6, delta = NULL, lambda = NULL, df = 10,...)
          {
            require(quantreg, quietly = TRUE)
            require(splines, quietly = TRUE)


            
            if(!is.null(delta) && is.null(lambda))
              lambda <- delta
            
            if(!is.null(lambda) && is.null(delta))
              delta <- lambda
              
            weights <- numeric()
            
            tau <- proportion


            
            logResSD <- object@data[["logResSD"]]
            Y <- object@data[["sqrtArrays"]]

            if(!is.null(delta))
              weights = (rank(logResSD)/length(logResSD)+delta)/lambda


            if(length(weights))
              fitted.values <- fitted(rq(Y ~ bs(logResSD,df=df),weights=weights,tau = tau))
            else
              fitted.values <- fitted(rq(Y ~ bs(logResSD,df=df),tau = tau))

            if(length(tau) > 1)
              {
                keep <- matrix(Y,ncol=ncol(fitted.values),nrow=nrow(fitted.values),
                               byrow=FALSE) > fitted.values
              }
            else
              {
                keep <- Y > fitted.values
                dim(keep) <- c(length(Y),1)
                dim(fitted.values) <- c(length(fitted.values),1)
              }
            ##keep <- drop(Y > fitted.values)

            ans <- object
            ans@keep <- keep
            ans@fitted <- fitted.values
            #ans@fitted <- drop(fitted.values)

            ans
            
            })
